this.name			= "PlanetFall";
this.author			= "Stranger, tightly based on Thargoid's PlanetFall v 1.51.1";
this.copyright		= "Creative Commons Attribution - Non-Commercial - Share Alike 3.0 license with clauses - see readme.txt.";
this.description	= "Enables planetary landing";
this.version		= "1.4";

//Main events

this.startUp = function()
	{ 
	this.lastPlanet = system.mainPlanet; // details of the last planet to be landed on, so we know what to move the player away from on launch
	this.planetFallLaunch = false; // as in 1.77 the alert condition is still set on launch - need a mask
	this.lastPlanetType = "Prime"; // what type of planet/moon we landed on
	this.messageBlock = false; // flag to stop request/cancel messages at launch from planet
	this.planetCount = 1; // default setting
	this.moonCount = 0; // default setting
	this.$systemScan(); // set up the initial planet and moon count
	this.planetFallOverride = false; // flag to allow other OXPs to override PlanetFall locations
	this.overrideRole = "default"; // if other OXPs want to ensure a specific role is used
	this.$checkForSpecialPlanets();
    this.gasGiantRadius = 100000;  // set filter fo giant planets
    this.planetTerranRadius = 40000; // set filter to planets with dense atmosphere
    this.$trafficState();
    this.sectorOnePlanetList = [0,1,2,3,4,7,8,10,11,13,16,17,18,19,20,21,23,28,29,35,36,39,42,44,46,50,55,62,66,73,86,89,90,93,99,100,101,111,118,124,126,129,131,132,141,147,150,153,154,172,177,186,188,198,200,221,222,227,228,241,246,250]; // Famous Planets Sector One
    this.sectorTwoPlanetList = [6,23,24,29,33,42,45,48,53,54,57,58,65,66,74,78,82,88,94,109,113,114,115,122,127,136,140,144,178,188,189,193,202,204,207,221,236]; // Famous Planets Sector Two

    this.maxPort = 10;
    this.landingChoice = 1;
    this.portList = new Array(
    "***", // fake port, newer selected & displayed
	"Космопорт",
    "Верфь",
    "Шахта",
    "Завод",
    "Гидроферма",
    "Научный комплекс",
    "Курорт",
    "Открытые поля",
    "Внешние OXP"

    );
    this.outputToConsole = true;

/* For convenience of mission writers, access for other OXPs is via "worldScripts.PlanetFall.lastPlanet" and "worldScripts.PlanetFall.lastPlanetType". 
Values for lastPlanetType are "Prime" for the main planet, "Sub" for every other planet, "Moon" for any moon and "Sun" / "GasGiant" for a sun or a Solar System OXP 
gas giant (for completeness - cannot land on either of the last two). */
	}

this.shipWillLaunchFromStation = function(station)
	{
	this.$updateview();
	if(station)
		{ 
		this.planetFallLaunch = true;
        player.ship.hudHidden = false;
		if(station.hasRole("planetFall_surface"))
			{
			this.fx = system.addVisualEffect("planetFall_launchFX",player.ship.position.add(player.ship.vectorForward.multiply(150)));
			this.fx.orientation = player.ship.orientation;
			}
		} 
	}

this.shipLaunchedFromStation = function(station)
	{
	if(EquipmentInfo.infoForKey("EQ_FRAME_SAVE") != null) // if launch is for Save Anywhere, don't want to move the player or nuke the station
		{ return; }

	if(station && station.hasRole("planetFall_surface"))
		{
		if(this.landingTimer) 
			{ delete this.landingTimer; }	
		player.commsMessage(system.name + "Планетарное управление - орбитальный импульс завершенн",5);
		player.commsMessage("Прощай командир " + player.name + ".",5);
		this.$removePort(station); // remove the surface-simulating station
		this.messageBlock = false;
		missionVariables.planetFallBreak += 0.001; // 0.1% cumulative chance of equipment failure 
		if(Math.random() < missionVariables.planetFallBreak) // landing equipment needs maintenance
			{
			player.ship.setEquipmentStatus("EQ_PLANETFALL", "EQUIPMENT_DAMAGED");
			player.consoleMessage("ТРЕВОГА - Планетарный посадочный модуль требует срочного технического обслуживания.", 10);
			player.consoleMessage("Не пытайтесь совершать посадку, прежде отремонтируйтесь!", 10);
			missionVariables.planetFallBreak = 0; // used equipment, no guarantee
			}
		}

	this.$systemScan();
	}

this.shipExitedWitchspace = function()
	{
	this.$checkForSpecialPlanets();
	this.overrideRole = "default";
	this.$systemScan();
    this.$trafficState();
	}

this.shipEnteredPlanetaryVicinity = function(planet)
	{
	if(!planet) { return; }

    this.$trafficState();
	this.lastPlanet = planet;
	if(!planet.isSun && !planet.hasOwnProperty("solarGasGiant") && !planet.hasOwnProperty("isGasGiant") && !planet.hasOwnProperty("PFNoLand") && !planet.hasOwnProperty("PFNoLandQuiet") && player.ship.equipmentStatus("EQ_PLANETFALL") == "EQUIPMENT_OK")
		{
		player.commsMessage(system.name + " Планетарная управления - прежде чем садиться, подойди и запроси разрешение на посадку.",5);
		}
	else
		{
		if(planet.isSun)
			{ player.consoleMessage("ТРЕВОГА - Подходя к Солнцу, важно контролиловать температуру обшивки!", 10); }
		if(planet.hasOwnProperty("solarGasGiant") || planet.hasOwnProperty("isGasGiant"))
			{ player.consoleMessage("ТРЕВОГА - Вы приближаетесь к газовому гиганту. Посадка здесь не возможна!", 10); }
		if(planet.hasOwnProperty("PFNoLand"))
			{ player.commsMessage("ТРЕВОГА - Посадка не возможно здесь, пожалуйста, поднимайтесь!", 10); }
		}
	}

this.shipExitedPlanetaryVicinity = function()
	{ 
	this.messageBlock = false; 
	this.shipDied(); // clean-up of any unremoved surface stations.
	}

this.alertConditionChanged = function()
	{
	if(this.planetFallLaunch)
		{
		this.planetFallLaunch = false;
		return;
		}

	if(player.alertAltitude && !this.lastPlanet.isSun && !this.lastPlanet.hasOwnProperty("solarGasGiant") && !this.lastPlanet.hasOwnProperty("PFNoLand") && !this.lastPlanet.hasOwnProperty("PFNoLandQuiet") && !this.lastPlanet.hasOwnProperty("isGasGiant"))
		{
		if(player.ship.equipmentStatus("EQ_PLANETFALL") != "EQUIPMENT_OK")
			{
			player.commsMessage(system.name + " Планетарная управления - Ваш корабль не оборудован для посадки.", 10);
			player.commsMessage("Не пытайтесь сесть, ваш корабль будет разрушен!", 10);
			}
		else
			{
			if(!this.messageBlock)
				{
				player.commsMessage(system.name + " Планетарное управление - запрос на посадку получен, ожидайте.", 6);
				player.commsMessage("Пожалуйста, дождитесь подтверждения, прежде чем начать заключительную стадию.", 6);
				}
            if(Math.random() > (0.75 - 0.5*missionVariables.planetFallBreak + 0.25*this.serviceLevel))// malfuncttion of homing beacon
                {
                this.homingLock = 0;
                }
			else
                {
                this.homingLock = 1;
                }
            this.clearanceDelay = 5 + (Math.ceil(Math.random()*15)); // delay between 6 and 20 seconds
			this.landingTimer = new Timer(this, this.$landingClearance, this.clearanceDelay);
            // a bit surprice...
            if((Math.random() > (0.85 + 0.02*system.government)) && (this.lastPlanet.radius < this.gasGiantRadius))
                {
                system.addShips("pirate", 4, player.ship.position, 15000);
                }
			}
		}

	if(player.alertAltitude && (this.lastPlanet.hasOwnProperty("solarGasGiant") || this.lastPlanet.hasOwnProperty("isGasGiant")))
		{ player.consoleMessage("ТРЕВОГА - Вы приближаетесь к газовому гиганту. Посадка не возможна!", 10); 	}

	if(player.alertAltitude && this.lastPlanet.hasOwnProperty("PFNoLand"))
		{ 
		player.consoleMessage("ТРЕВОГА - Посадка не возможна!", 10); 
		return;
		}
		
	if(!player.alertAltitude && this.landingTimer && this.landingTimer.isRunning && !player.ship.docked && EquipmentInfo.infoForKey("EQ_FRAME_SAVE") == null)
		{
		this.landingTimer.stop();
		delete this.landingTimer;
		missionVariables.planetFallBreak += 0.001; // 0.1% cumulative chance of equipment failure
		if(!this.messageBlock)
			{
			player.commsMessage(system.name + " Планетарное управление - оформление запроса на посадку отменено.", 6);
			player.commsMessage("Прощай командир " + player.name + ".", 6);
			this.messageBlock = false;
			}
		}
	}

this.shipApproachingPlanetSurface = function(planet) 
	{
	if(!planet || player.ship.equipmentStatus("EQ_PLANETFALL") != "EQUIPMENT_OK" || planet.hasOwnProperty("PFNoLandQuiet"))
		{
		return;
		}

	if(planet.hasOwnProperty("PFNoLand"))
		{
		player.commsMessage(system.name + " Планетарное управление - Срочное предупреждение - посадка не возможна!", 6);
		return;	
		}	

    if(this.landingChoice != 9)
        {
        this.planetFallOverride = false;
        }
	
	this.lastPlanet = planet;	

    if(this.lastPlanet.radius >= this.gasGiantRadius)
        {
        this.landingChoice = 6;
        }

	if(!this.landingTimer)  
		{
		if(!planet.isSun && !planet.hasOwnProperty("solarGasGiant") && !planet.hasOwnProperty("isGasGiant"))
			{
			this.messageBlock = true;
			if(planet === system.mainPlanet)
				{
				this.lastPlanetType = "Prime";
				if(this.planetFallOverride)
					{
					if(this.overrideRole === "default")
						{ this.overrideRole = "planetFall_mainSurface_externalOXP" };
					this.port = player.ship.spawnOne(this.overrideRole);
					this.$turnStation(this.port);
					this.overrideRole = "default";
					}
				else                
					{
					if(this.landingChoice == 1) // spawn capital city
						{
						this.port = player.ship.spawnOne(this.mainPort1); 
						this.$turnStation(this.port);
						}
                    if(this.landingChoice == 2) // spawn a shipyard
                        {
						this.port = player.ship.spawnOne(this.mainPort2); 
						this.$turnStation(this.port);
                        }
                    if(this.landingChoice == 3) // spawn a mine
                        {
						this.port = player.ship.spawnOne(this.mainPort3); 
						this.$turnStation(this.port);
                        }
					if(this.landingChoice == 4) // spawn factory
						{
						this.port = player.ship.spawnOne(this.mainPort4); 
						this.$turnStation(this.port);
						}
					if(this.landingChoice == 5) // spawn farm
						{
						this.port = player.ship.spawnOne(this.mainPort5); 
						this.$turnStation(this.port);
						}
                    if(this.landingChoice == 6) // spawn a research complex
						{
						this.port = player.ship.spawnOne(this.mainPort6); 
						this.$turnStation(this.port);
						}
					if(this.landingChoice == 7) // spawn leisure complex
						{
						this.port = player.ship.spawnOne(this.mainPort7); 
						this.$turnStation(this.port);
						}
					if((this.landingChoice == 8)||(this.homingLock == 0)) // spawn open fields
						{
						this.port = player.ship.spawnOne(this.mainPort8); 
						this.$turnStation(this.port);
						}
					if((this.landingChoice == 9)||(Math.random() > 0.90)) // spawn external OXP location, or if none then Capital City
						{
						if(this.overrideRole !== "default") 
							{ this.port = player.ship.spawnOne(this.overrideRole); }
							else
							{ this.port = player.ship.spawnOne("planetFall_mainSurface_externalOXP"); }
						if(!this.port)
							{
							log(this.name, "Main surface external spawn failure");
							this.port = player.ship.spawnOne(this.mainPort1); 
							}					
						this.$turnStation(this.port);
						this.overrideRole = "default";
						}
					}
					
				if(this.lastPlanet.name)
					{ this.port.displayName = this.lastPlanet.name + " - " + this.port.displayName; }
				else
					{ this.port.displayName = system.name + " Prime - " + this.port.displayName; } // set the name to the surface of the prime planet in the system
				}
			else
				{
				if(planet.hasAtmosphere)
					{
					this.lastPlanetType = "Sub";
					if(this.planetFallOverride)
						{
						if(this.overrideRole == "default") { this.overrideRole = "planetFall_subSurface_externalOXP"; };
						this.port = player.ship.spawnOne(this.overrideRole);
						this.$turnStation(this.port);
						this.overrideRole = "default";
						}
					else
						{ 
						if(this.landingChoice == 1) // spawn main port
                            {
                            this.port = player.ship.spawnOne(this.minorPort1); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 2) // spawn a shipyard
							{
                            this.port = player.ship.spawnOne(this.minorPort2); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 3) // spawn a mine
                            {
                            this.port = player.ship.spawnOne(this.minorPort3); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 4) // spawn a factory
                            {
                            this.port = player.ship.spawnOne(this.minorPort4); 
                            this.$turnStation(this.port);                           
                            }
						if(this.landingChoice == 5) // spawn a farm
                            {
                            this.port = player.ship.spawnOne(this.minorPort5); 
                            this.$turnStation(this.port);                            
                            }
						if(this.landingChoice == 6) // spawn a research complex
                            {
                            this.port = player.ship.spawnOne(this.minorPort6); 
                            this.$turnStation(this.port);                            
                            }
						if(this.landingChoice == 7) // spawn a leisure complex
							{
                            this.port = player.ship.spawnOne(this.minorPort7); 
                            this.$turnStation(this.port);
                            }
                        if((this.landingChoice == 8)||(this.homingLock == 0)) // spawn open fields
                            {
                            this.port = player.ship.spawnOne(this.minorPort8); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 9) // spawn external OXP location, or if none then a research complex
							{
							if(this.overrideRole !== "default") 
								{ this.port = player.ship.spawnOne(this.overrideRole); }
								else
								{ this.port = player.ship.spawnOne("planetFall_subSurface_externalOXP"); }
							if(!this.port)
								{
								log(this.name, "Sub surface external spawn failure");
								this.port = player.ship.spawnOne(this.minorPort6);
								}	
                            this.$turnStation(this.port);
							this.overrideRole = "default";
							}
                            
                        if(this.lastPlanet.radius >= this.gasGiantRadius)
                            {
                            this.port = player.ship.spawnOne(this.minorPort6); 
                            this.$turnStation(this.port);
                            }
						}

					if(this.lastPlanet.name)
						{ this.port.displayName = this.lastPlanet.name + " - " + this.port.displayName; }
					else
						{ this.port.displayName = system.name + " Minor - " + this.port.displayName; } // set the name to just the surface, so we can identify easily the main planet
					}
				else
					{
					this.lastPlanetType = "Moon";
					if(this.planetFallOverride)
						{
						if(this.overrideRole == "default") { this.overrideRole = "planetFall_moonSurface_externalOXP" };
						this.port = player.ship.spawnOne(this.overrideRole);
						this.$turnStation(this.port);
						this.overrideRole = "default";
						}
					else
						{
						if(this.landingChoice == 1) // spawn main port
							{
                            this.port = player.ship.spawnOne(this.moonPort1); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 2) // spawn a shipyard
							{
                            this.port = player.ship.spawnOne(this.moonPort2); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 3) // spawn a mine
                            {
                            this.port = player.ship.spawnOne(this.moonPort3); 
                            this.$turnStation(this.port);                           
                            }
						if(this.landingChoice == 4) // spawn a factory
                            {
                            this.port = player.ship.spawnOne(this.moonPort4); 
                            this.$turnStation(this.port);                           
                            }
						if(this.landingChoice == 5) // spawn a farm
                            {
                            this.port = player.ship.spawnOne(this.moonPort5); 
                            this.$turnStation(this.port);                            
                            }
						if(this.landingChoice == 6) // spawn a research complex
							{
                            this.port = player.ship.spawnOne(this.moonPort6); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 7) // spawn a leisure complex
							{
                            this.port = player.ship.spawnOne(this.moonPort7); 
                            this.$turnStation(this.port);
                            }
                        if((this.landingChoice == 8)||(this.homingLock == 0)) // spawn open fields
                            {
                            this.port = player.ship.spawnOne(this.moonPort8); 
                            this.$turnStation(this.port);
                            }
						if(this.landingChoice == 9) // spawn external OXP location, or if none then a research complex
							{
							if(this.overrideRole !== "default") 
								{ this.port = player.ship.spawnOne(this.overrideRole); }
								else
								{ this.port = player.ship.spawnOne("planetFall_moonSurface_externalOXP"); }
							if(!this.port)
								{
								log(this.name, "Moon external spawn failure");
								this.port = player.ship.spawnOne(this.moonPort6); 
								}	
							this.$turnStation(this.port);
							this.overrideRole = "default";
							}
						}

					if(this.lastPlanet.name)
						{ this.port.displayName = this.lastPlanet.name + " - " + this.port.displayName; }
					else
						{ this.port.displayName = "Moon of " + system.name + " - " + this.port.displayName; } // set the name to the moon, so we can identify it as a moon
					}
				}
			this.fx = system.addVisualEffect("planetFall_landingFX",player.ship.position.add(player.ship.vectorForward.multiply(170)));	
			this.port.dockPlayer();	
			}
		else
			{
			if(planet.hasOwnProperty("solarGasGiant") || planet.hasOwnProperty("isGasGiant"))
				{
				this.lastPlanetType = "GasGiant";
				}
			else
				{
				this.lastPlanetType = "Sun";
				}
			}
		}
	else
		{ // if the PF landing timer is still running
		player.commsMessage(system.name + " Планетарное управление - Поднимайтесь вверх, у вас нет разрешения на посадку!", 6);
		player.commsMessage("Вернитесь на орбиту и попробуйте еще раз.", 6);
		this.landingTimer.stop();
		delete this.landingTimer;
		}
	}

this.shipWillDockWithStation = function(station)
	{
	if(station.hasRole("planetFall_surface") || station.hasRole("planetFall_mainSurface_externalOXP") || station.hasRole("planetFall_subSurface_externalOXP") || station.hasRole("planetFall_moonSurface_externalOXP") )
		{
		if(!this.fx) { this.fx = system.addVisualEffect("planetFall_landingFX",player.ship.position.add(player.ship.vectorForward.multiply(170))); }
		this.selectTexture = 1 + (Math.floor(Math.random() * 9));
		if(station.hasRole("planetFall_moonSurface"))
			{ this.fx.setMaterials({"planetFall_planet1.png": {diffuse_map: "planetFall_moon" + this.selectTexture + ".png" }}); }
		else
			{ this.fx.setMaterials({"planetFall_planet1.png": {diffuse_map: "planetFall_planet" + this.selectTexture + ".png" }}); }
        }
	}

this.shipDockedWithStation = function(station)
    {
    if(this.landingChoice == 9)
        { return; }
    if(station.hasRole("planetFall_surface"))
        {
        this.$portImage();
        player.ship.hudHidden = true;
        }
    else
        {
        player.ship.hudHidden = false;
        }    

    // main planet - functional ports
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_capitalCity"))
        {
        this.portTitle = "Столица";
        this.portImageCaptured = this.mainCityImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_shipYard"))
        {
        this.portTitle = "Верфь";
        this.portImageCaptured = this.mainShipYardImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_mine"))
        {
        this.portTitle = "Шахта";
        this.portImageCaptured = this.mainMineImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_factory"))
        {
        this.portTitle = "Завод";
        this.portImageCaptured = this.mainFactoryImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_farm"))
        {
        this.portTitle = "Гидроферма";
        this.portImageCaptured = this.mainFarmImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_researchComplex"))
        {
        this.portTitle = "Научный комплекс";
        this.portImageCaptured = this.mainLabImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_leisureComplex"))
        {
        this.portTitle = "Курорт";
        this.portImageCaptured = this.mainLeisureImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }

    // main planet - fields
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_fields"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Открытые поля";
        this.portImageCaptured = this.mainFieldImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }

    // main planet - obsolete & auxiliary non-trading ports
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_auxiliaryShipYard"))
        {
        this.portTitle = "Старая верфь";
        this.portImageCaptured = this.mainAuxiliaryShipYardImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_auxiliaryFactory"))
        {
        this.portTitle = "Старый завод";
        this.portImageCaptured = this.mainAuxiliaryFactoryImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_auxiliaryFarm"))
        {
        this.portTitle = "Вспомогательная гидроферма";
        this.portImageCaptured = this.mainAuxiliaryFarmImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }

    // main planet - derelict ports
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictCity"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный город";
        this.portImageCaptured = this.mainDerelictCityImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictShipYard"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная верфь";
        this.portImageCaptured = this.mainDerelictShipYardImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictMine"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная шахта";
        this.portImageCaptured = this.mainDerelictMineImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictFactory"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный завод";
        this.portImageCaptured = this.mainDerelictFactoryImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictFarm"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная ферма";
        this.portImageCaptured = this.mainDerelictFarmImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictResearchComplex"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный научный комплекс";
        this.portImageCaptured = this.mainDerelictLabImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictLeisureComplex"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный курорт";
        this.portImageCaptured = this.mainDerelictLeisureImage;
        this.fieldImageCaptured = this.mainFieldImage;
        }

    // minor planet - ports
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_colony"))
        {
        this.portTitle = "Колониальный город";
        this.portImageCaptured = this.minorCityImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_shipYard"))
        {
        this.portTitle = "Верфь";
        this.portImageCaptured = this.minorShipYardImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_mine"))
        {
        this.portTitle = "Шахта";
        this.portImageCaptured = this.minorMineImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_factory"))
        {
        this.portTitle = "Завод";
        this.portImageCaptured = this.minorFactoryImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_farm"))
        {
        this.portTitle = "Гидроферма";
        this.portImageCaptured = this.minorFarmImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_researchComplex"))
        {
        this.portTitle = "Научный комплекс";
        this.portImageCaptured = this.minorLabImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_leisureComplex"))
        {
        this.portTitle = "Курорт";
        this.portImageCaptured = this.minorLeisureImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }

    // minor planet - fields
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_fields"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Открытые поля";
        this.portImageCaptured = this.minorFieldImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }

    // minor planet - derelict ports
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictColony"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный колониальный город";
        this.portImageCaptured = this.minorDerelictCityImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictShipYard"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная верфь";
        this.portImageCaptured = this.minorDerelictShipYardImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictMine"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная шахта";
        this.portImageCaptured = this.minorDerelictMineImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictFactory"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный завод";
        this.portImageCaptured = this.minorDerelictFactoryImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictFarm"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная гидроферма";
        this.portImageCaptured = this.minorDerelictFarmImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictLeisureComplex"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенный курорт";
        this.portImageCaptured = this.minorDerelictLeisureImage;
        this.fieldImageCaptured = this.minorFieldImage;
        }

    // moon - ports
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_dome"))
        {
        this.portTitle = "Колониальный купол";
        this.portImageCaptured = this.moonCityImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_shipYard"))
        {
        this.portTitle = "Верфь";
        this.portImageCaptured = this.moonShipYardImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_mine"))
        {
        this.portTitle = "Шахта";
        this.portImageCaptured = this.moonMineImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_factory"))
        {
        this.portTitle = "Автоматический завод";
        this.portImageCaptured = this.moonFactoryImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_farm"))
        {
        this.portTitle = "Тепличный купол";
        this.portImageCaptured = this.moonFarmImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_researchComplex"))
        {
        this.portTitle = "Научный комплекс";
        this.portImageCaptured = this.moonLabImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_leisureDome"))
        {
        this.portTitle = "Курортный купол";
        this.portImageCaptured = this.moonLeisureImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }

    // moon - fields
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_fields"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Лунное поле";
        this.portImageCaptured = this.moonFieldImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }

    // moon - derelict ports
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictDome"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный колониальный купол";
        this.portImageCaptured = this.moonDerelictColonyImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictShipYard"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная верфь";
        this.portImageCaptured = this.moonDerelictShipYardImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictMine"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Заброшенная шахта";
        this.portImageCaptured = this.moonDerelictMineImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictFactory"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный автоматический завод";
        this.portImageCaptured = this.moonDerelictFactoryImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictFarm"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный тепличный купол";
        this.portImageCaptured = this.moonDerelictFarmImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }
    if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictLeisureDome"))
        {
        missionVariables.planetFallBreak += 0.001; // additional tear and wear
        this.portTitle = "Забошенный курортный купол";
        this.portImageCaptured = this.moonDerelictLeisureImage;
        this.fieldImageCaptured = this.moonFieldImage;
        }

    if(station.hasRole("planetFall_surface"))
        {
        mission.runScreen({
            title: this.portTitle,
            background: {name: this.portImageCaptured, height: 480 },
            }
            );
        }

    }

this.shipLeavingPlanetSurface = function(planet)
	{
	this.lastPlanet = planet;
	this.messageBlock = false;
	}

this.guiScreenChanged = function(to,from)
	{
	if(!player.ship.docked) // for GUI screen changes whilst in flight, which we can ignore
		{ return; }

	if(!player.ship.dockedStation.hasRole("planetFall_surface")) // if we're at a trunk or other OXP's station
		{ return; }

    if(this.landingChoice == 9) // if we're at a trunk or surface ports of external OXP's such as Feudal States
        { return; }
        
    if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET" || guiScreen == "GUI_SCREEN_SHIPYARD")
        {
        player.ship.hudHidden = false;
        }
    else
        {
        player.ship.hudHidden = true;
        }

    if (to === "GUI_SCREEN_SYSTEM_DATA" && from === "GUI_SCREEN_STATUS")
        {
        mission.runScreen({
            title: "Планетарный вид",
            background: {name: this.fieldImageCaptured, height: 480 },});       
        }

    if (to === "GUI_SCREEN_SYSTEM_DATA" && from === "GUI_SCREEN_MARKET")
        {
        mission.runScreen({
            title: "Портовый вид",
            background: {name: this.portImageCaptured, height: 480 },});       
        }

	if(!player.ship.dockedStation.hasRole("planetFall_noTrade")) // if we're at a location where trading is possible
		{ return; }

	 // no-one to trade with in empty fields
	if(player.ship.dockedStation.hasRole("planetFall_mainSurface_fields")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Открытые поля",
                messageKey: "planetFall_noTrade",
                background: {name: this.mainFieldImage, height: 480 },});
			}
		}

	 // no-one to trade with in auxiliary OR derelict object
    if(this.warState == 0)
        {    
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_auxiliaryShipYard")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
                {
                mission.runScreen({
                    title: "Старая верфь",
                    messageKey: "planetFall_shipYardAuxiliary",
                    background: {name: this.mainAuxiliaryShipYardImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictMine")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
                {
                mission.runScreen({
                    title: "Заброшенная шахта",
                    messageKey: "planetFall_mineDerelict",
                    background: {name: this.mainDerelictMineImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_auxiliaryFactory")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
                {
                mission.runScreen({
                    title: "Старый завод",
                    messageKey: "planetFall_factoryAuxiliary",
                    background: {name: this.mainAuxiliaryFactoryImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_auxiliaryFarm")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
                {
                mission.runScreen({
                    title: "Вспомогательная гидроферма",
                    messageKey: "planetFall_farmAuxiliary",
                    background: {name: this.mainAuxiliaryFarmImage, height: 480 },});
                }
            }
        }

    // wartime is a hard time for regular trade, but sometimes your help is needed!
    if (this.warState == 1)
        {

        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictShipYard")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP")
                {
                mission.runScreen({
                    title: "Заброшенная верфь",
                    messageKey: "planetFall_shipYardDerelictWar",
                    background: {name: this.mainDerelictShipYardImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictMine")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP")
                {
                mission.runScreen({
                    title: "Заброшенная шахта",
                    messageKey: "planetFall_mineDerelictWar",
                    background: {name: this.mainDerelictMineImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictFactory")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP")
                {
                mission.runScreen({
                    title: "Забошенный завод",
                    messageKey: "planetFall_factoryDerelictWar",
                    background: {name: this.mainDerelictFactoryImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictFarm")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP")
                {
                mission.runScreen({
                    title: "Заброшенная ферма",
                    messageKey: "planetFall_farmDerelictWar",
                    background: {name: this.mainDerelictFarmImage, height: 480 },});
                }
            }

        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictResearchComplex")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
                {
                mission.runScreen({
                    title: "Забошенный научный комплекс",
                    messageKey: "planetFall_researchDerelictWar",
                    background: {name: this.mainDerelictLabImage, height: 480 },});
                }
            }
        if(player.ship.dockedStation.hasRole("planetFall_mainSurface_derelictLeisureComplex")) 
            {
            if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
                {
                mission.runScreen({
                    title: "Забошенный курорт",
                    messageKey: "planetFall_leisureDerelictWar",
                    background: {name: this.mainDerelictLeisureImage, height: 480 },});
                }
            }
        }

	// no-one to trade with in empty fields
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_fields")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET" || guiScreen == "GUI_SCREEN_SHIPYARD")
			{
			mission.runScreen({
            title:"Открытые поля",
            messageKey:"planetFall_noTrade",
            background: {name: this.minorFieldImage, height: 480 },});
			}
		}

	 // no-one to trade with in derelict object
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictColony")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Забошенный колониальный город",
                messageKey: "planetFall_minorColonyDerelict",
                background: {name: this.minorDerelictCityImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictShipYard")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенная верфь",
                messageKey: "planetFall_minorShipYardDerelict",
                background: {name: this.minorDerelictShipYardImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictMine")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенная шахта",
                messageKey: "planetFall_minorMineDerelict",
                background: {name: this.minorDerelictMineImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictFactory")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Забошенный завод",
                messageKey: "planetFall_minorFactoryDerelict",
                background: {name: this.minorDerelictFactoryImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictFarm")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенная гидроферма",
                messageKey: "planetFall_minorFarmDerelict",
                background: {name: this.minorDerelictFarmImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_subSurface_derelictLeisureComplex")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Забошенный курорт",
                messageKey: "planetFall_minorLeisureDerelict",
                background: {name: this.minorDerelictLeisureImage, height: 480 },});
			}
		}

	// no-one to trade with on the empty lunar surface
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_fields") ) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
			mission.runScreen({
                title:"Открытые поля",
                messageKey:"planetFall_noTrade",
                background: {name: this.moonFieldImage, height: 480 },});
			}
		}

	 // no-one to trade with in derelict object
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictDome")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Забошенный колониальный купол",
                messageKey: "planetFall_moonColonyDerelict",
                background: {name: this.moonDerelictColonyImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictShipYard")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенная верфь",
                messageKey: "planetFall_moonShipYardDerelict",
                background: {name: this.moonDerelictShipYardImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictMine")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенная шахта",
                messageKey: "planetFall_moonMineDerelict",
                background: {name: this.moonDerelictMineImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictFactory")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенный автоматический завод",
                messageKey: "planetFall_moonFactoryDerelict",
                background: {name: this.moonDerelictFactoryImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictFarm")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенный тепличный купол",
                messageKey: "planetFall_moonFarmDerelict",
                background: {name: this.moonDerelictFarmImage, height: 480 },});
			}
		}
	if(player.ship.dockedStation.hasRole("planetFall_moonSurface_derelictLeisureDome")) 
		{
		if(guiScreen == "GUI_SCREEN_EQUIP_SHIP" || guiScreen == "GUI_SCREEN_MARKET")
			{
            mission.runScreen({
                title: "Заброшенный курортный купол",
                messageKey: "planetFall_moonLeisureDerelict",
                background: {name: this.moonDerelictLeisureImage, height: 480 },});
			}
		}
	}

this.playerBoughtEquipment = function(equipment)  // guarantee first ten launches won't break equipment
	{
	if(equipment == "EQ_PLANETFALL")
		{ missionVariables.planetFallBreak = -0.01; }
	}

this.$landingClearance = function()
	{
	this.landingTimer.stop();
	delete this.landingTimer;
	player.commsMessage(system.name + " Планетарное управление - разрешение на посадку получено.",6);
	player.commsMessage("Пожалуйста, приближайтесь к поверхности медленно, следите за температурой обшивки.", 6);
	}

this.shipDied = function()
	{
	let ports = system.shipsWithRole("planetFall_surface"); // an array of any rogue "surface stations" in the system
	this.portCount = ports.length; // how many rogues we have that need to be removed
	if(this.portCount > 0) for (let i=0; i<this.portCount; i++) // if there are any, loop through and remove them
		{ this.$removePort(ports[i]); }
	if(this.landingTimer && this.landingTimer.isRunning)
		{
		this.landingTimer.stop();
		delete this.landingTimer;	
		}
	}

this.$systemScan = function()
	{
	function isMoon(entity) {return (entity.isPlanet && !entity.hasAtmosphere) };
	this.systemArray = system.planets;
	this.systemCount = this.systemArray.length;
	this.moonCount = system.filteredEntities(this.systemArray, isMoon).length;
	this.planetCount = this.systemCount - this.moonCount;
	}

this.$removePort = function(port)
	{	
	if(port.hasRole("planetFall_surface"))
		{ port.remove(); };
	}

this.$checkForSpecialPlanets = function()
	{ this.scriptDelay = new Timer(this, this.$setFlags, 0.25); }

this.$setFlags = function()
	{	
	if(galaxyNumber === 0 && system.ID === 246) // if we're at Tianve (G0, S227), set up the pulsar
		{
		for (let i=0; i<system.planets.length; i++) 
			{ 
			if(system.planets[i].texture === "tianve_pulsar.png") 
				{ system.planets[i].PFNoLand = true; } 
			}
		}
	}

this.$turnStation = function(tempStation)
	{
	if(!tempStation) { return; } // just in case the station doesn't survive spawning.
	let stationVector = tempStation.position.subtract(this.lastPlanet.position).direction(); // unit vector pointing away from the planet
	let angle = tempStation.heading.angleTo(stationVector); // angle between current heading and target heading
	let cross = tempStation.heading.cross(stationVector).direction(); // set the plane where we should rotate in
	tempStation.orientation = tempStation.orientation.rotate(cross, -angle); // re-orient the station away from the planet
	tempStation.position = tempStation.position.add(stationVector.multiply(3000));
	tempStation.breakPattern = false;
	}

// set war & traffic state
this.$trafficState = function()
    {
    var planetInfo = system.description;
    var warKeyword1 = "civil";
    this.warFlag1 = planetInfo.indexOf(warKeyword1);    
    var warKeyword2 = "war";
    this.warFlag2 = planetInfo.indexOf(warKeyword2);
    if((this.warFlag1 != -1) && (this.warFlag2 != -1))
        {
        this.warFlag = 1;
        }
    else
        {
        this.warFlag = 0;
        }
    this.warLevel = 0.75 + 0.05*system.government + 0.025*system.techLevel - 0.5*this.warFlag; // threshold of war conflicts (high value - reduced risk)
    if(Math.random() >  this.warLevel)
        {
        this.warState = 1;
        }
    else
        {
        this.warState = 0;
        }
    this.serviceLevel = 0.05*system.techLevel + 0.05*system.government; // level of ground control competence - reduced in low-tech unstable systems
    if(this.warState == 1)
        {
        this.serviceLevel = 0;// service dropped in wartime
        }
    this.$selectPort();
    }

// set port type
this.$selectPort = function()
    {

    // main ports
    // civil war - affects only main planet ports
    if(this.warState == 1)
        {
        this.mainPort1 = "planetFall_mainSurface_derelictCity";
        this.mainPort2 = "planetFall_mainSurface_derelictShipYard";
        this.mainPort3 = "planetFall_mainSurface_derelictMine";
        this.mainPort4 = "planetFall_mainSurface_derelictFactory";
        this.mainPort5 = "planetFall_mainSurface_derelictFarm";
        this.mainPort6 = "planetFall_mainSurface_derelictResearchComplex";
        this.mainPort7 = "planetFall_mainSurface_derelictLeisureComplex";
        }
    // regular case
    else
        {
        // capital city
        this.mainPort1 = "planetFall_mainSurface_capitalCity";
        // shipyard
        if(system.techLevel >= system.economy)
            {
            this.mainPort2 = "planetFall_mainSurface_shipYard";
            }        
        else
            {
            this.mainPort2 = "planetFall_mainSurface_auxiliaryShipYard";
            }
        // mine
        if((system.techLevel < 4) || ((system.techLevel < 10) && (system.economy < 3)))
            {
            this.mainPort3 = "planetFall_mainSurface_mine";
            }        
        else
            {
            this.mainPort3 = "planetFall_mainSurface_derelictMine";
            }
        // factory
        if(system.economy < 5)
            {
            this.mainPort4 = "planetFall_mainSurface_factory";
            }        
        else
            {
            this.mainPort4 = "planetFall_mainSurface_auxiliaryFactory";
            }
        // farm
        if(system.economy > 2)
            {
            this.mainPort5 = "planetFall_mainSurface_farm";
            }        
        else
            {
            this.mainPort5 = "planetFall_mainSurface_auxiliaryFarm";
            }
        // research complex
        this.mainPort6 = "planetFall_mainSurface_researchComplex";
        // leisure complex
        this.mainPort7 = "planetFall_mainSurface_leisureComplex";
        }
    // open fields
    this.mainPort8 = "planetFall_mainSurface_fields";

    // minor ports - colony complex
    if(system.techLevel > system.economy)
        {
        this.minorPort1 = "planetFall_subSurface_colony";
        this.minorPort2 = "planetFall_subSurface_shipYard";
        this.minorPort3 = "planetFall_subSurface_mine";
        this.minorPort4 = "planetFall_subSurface_factory";
        this.minorPort5 = "planetFall_subSurface_farm";
        this.minorPort7 = "planetFall_subSurface_leisureComplex";
        }
    else
        {
        this.minorPort1 = "planetFall_subSurface_derelictColony";        
        this.minorPort2 = "planetFall_subSurface_derelictShipYard";
        this.minorPort3 = "planetFall_subSurface_derelictMine";
        this.minorPort4 = "planetFall_subSurface_derelictFactory";
        this.minorPort5 = "planetFall_subSurface_derelictFarm";
        this.minorPort7 = "planetFall_subSurface_derelictLeisureComplex";
        }
    // research complex
    this.minorPort6 = "planetFall_subSurface_researchComplex";   
    // open fields
    this.minorPort8 = "planetFall_subSurface_fields";

    // moon ports - colony complex
    if(system.techLevel > system.economy)
        {
        this.moonPort1 = "planetFall_moonSurface_dome";
        this.moonPort2 = "planetFall_moonSurface_shipYard";
        this.moonPort3 = "planetFall_moonSurface_mine";
        this.moonPort4 = "planetFall_moonSurface_factory";
        this.moonPort5 = "planetFall_moonSurface_farm";
        this.moonPort7 = "planetFall_moonSurface_leisureDome";
        }
    else
        {
        this.moonPort1 = "planetFall_moonSurface_derelictDome";        
        this.moonPort2 = "planetFall_moonSurface_derelictShipYard"; 
        this.moonPort3 = "planetFall_moonSurface_derelictMine";
        this.moonPort4 = "planetFall_moonSurface_derelictFactory";
        this.moonPort5 = "planetFall_moonSurface_derelictFarm";
        this.moonPort7 = "planetFall_moonSurface_derelictLeisureDome";
        }    
    // research complex
    this.moonPort6 = "planetFall_moonSurface_researchComplex";  
    // open fields
    this.moonPort8 = "planetFall_moonSurface_fields";

    }

// set planet port view
this.$portImage = function()
    {
    var cityImageShift = 0;
    if((system.government == 0) || (system.government == 2))
        {
    cityImageShift = 1;
        }
    if((system.government == 3) || (system.government == 4))
        {
    cityImageShift = 2;
        }
    if(system.government > 4)
        {
    cityImageShift = 3;
        }
    // define tech level
    if(system.techLevel < 4)
        {
        this.techFlag = "lowTech";
        }
    else
        {
        if(system.techLevel < 7)
            {
            this.techFlag = "midTech";
            }
        else
            {
            if(system.techLevel < 10)
                {
                this.techFlag = "advanceTech";
                }
            else
                {
                this.techFlag = "highTech";
                }
            }
        }
    // define economy
    if(system.economy < 3)
        {
        this.economyFlag = "industry";
        }
    else
        {
        if(system.economy < 5)
            {
            this.economyFlag = "mixed";
            }
        else
            {
            this.economyFlag = "agro";
            }
        }

    // main ports
    if(this.techFlag == "lowTech")
        {
        this.mainCityImage = "mainCity" + (1 + cityImageShift) + ".png";
        this.mainMineImage = "mainMine1.png";
        this.mainFarmImage = "mainFarm1.png";
        this.mainLabImage = "mainLab1.png";
        this.mainLeisureImage = "mainLeisure1.png";
        }
    if((this.techFlag == "midTech") && (this.economyFlag == "agro"))
        {
        this.mainCityImage = "mainCity" + (5 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard1.png";
        this.mainFarmImage = "mainFarm2.png";
        this.mainLabImage = "mainLab2.png";
        this.mainLeisureImage = "mainLeisure2.png";
        }
    if((this.techFlag == "midTech") && (this.economyFlag == "mixed"))
        {
        this.mainCityImage = "mainCity" + (9 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard1.png";
        this.mainFactoryImage = "mainFactory1.png";
        this.mainFarmImage = "mainFarm3.png";
        this.mainLabImage = "mainLab3.png";
        this.mainLeisureImage = "mainLeisure3.png";
        }
    if((this.techFlag == "midTech") && (this.economyFlag == "industry"))
        {
        this.mainCityImage = "mainCity" + (13 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard2.png";
        this.mainMineImage = "mainMine2.png";
        this.mainFactoryImage = "mainFactory2.png";
        this.mainLabImage = "mainLab4.png";
        this.mainLeisureImage = "mainLeisure4.png";
        }
    if((this.techFlag == "advanceTech") && (this.economyFlag == "agro"))
        {
        this.mainCityImage = "mainCity" + (17 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard3.png";
        this.mainFarmImage = "mainFarm4.png";
        this.mainLabImage = "mainLab5.png";
        this.mainLeisureImage = "mainLeisure5.png";
        }
    if((this.techFlag == "advanceTech") && (this.economyFlag == "mixed"))
        {
        this.mainCityImage = "mainCity" + (21 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard4.png";
        this.mainFactoryImage = "mainFactory3.png";
        this.mainFarmImage = "mainFarm5.png";
        this.mainLabImage = "mainLab6.png";
        this.mainLeisureImage = "mainLeisure6.png";
        }
    if((this.techFlag == "advanceTech") && (this.economyFlag == "industry"))
        {
        this.mainCityImage = "mainCity" + (25 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard5.png";
        this.mainMineImage = "mainMine3.png";
        this.mainFactoryImage = "mainFactory4.png";
        this.mainLabImage = "mainLab7.png";
        this.mainLeisureImage = "mainLeisure7.png";
        }
    if((this.techFlag == "highTech") && (this.economyFlag == "mixed"))
        {
        this.mainCityImage = "mainCity" + (28 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard6.png";
        this.mainFactoryImage = "mainFactory5.png";
        this.mainFarmImage = "mainFarm6.png";
        this.mainLabImage = "mainLab8.png";
        this.mainLeisureImage = "mainLeisure8.png";
        }
    if((this.techFlag == "highTech") && (this.economyFlag == "industry"))
        {
        this.mainCityImage = "mainCity" + (31 + cityImageShift) + ".png";
        this.mainShipYardImage = "mainShipYard7.png";
        this.mainFactoryImage = "mainFactory6.png";
        this.mainLabImage = "mainLab9.png";
        this.mainLeisureImage = "mainLeisure9.png";
        }
    if((this.techFlag == "lowTech") && (system.info.sun_radius < 275000))
        {
        this.mainCityImage = "mainCity35.png";
        this.mainFarmImage = "mainFarm7.png";
        this.mainLabImage = "mainLab10.png";
        this.mainLeisureImage = "mainLeisure10.png";
        }   

        // main auxiliary & derelict ports
        this.mainAuxiliaryShipYardImage = "mainAuxiliaryShipYard.png";
        this.mainAuxiliaryFactoryImage = "mainAuxiliaryFactory.png";
        this.mainAuxiliaryFarmImage = "mainAuxiliaryFarm.png";
        this.mainDerelictCityImage = "mainDerelictCity.png";
        this.mainDerelictShipYardImage = "mainDerelictShipYard.png";
        this.mainDerelictMineImage = "mainDerelictMine.png";
        this.mainDerelictFarmImage = "mainDerelictFarm.png";
        this.mainDerelictLabImage = "mainDerelictLab.png";
        this.mainDerelictLeisureImage = "mainDerelictLeisure.png";
        if(system.techLevel < 7)
            {
            this.mainDerelictFactoryImage = "mainDerelictFactory1.png";            
            }
        else
            {
            this.mainDerelictFactoryImage = "mainDerelictFactory2.png";            
            }

    // Famous Planets landscapes
    // Sector One
    if(system.info.galaxyID == 0)
        {
        var countSectorOne = this.sectorOnePlanetList.length;
        var flagSectorOne = -1;
        for (let i=0; i<countSectorOne; i++) 
            { 
            if(this.sectorOnePlanetList[i] == system.info.systemID)
                {
                flagSectorOne = this.sectorOnePlanetList[i];
                }
            }
        }
    // Sector Two
    if(system.info.galaxyID == 1)
        {
        var countSectorTwo = this.sectorTwoPlanetList.length;
        var flagSectorTwo = -1;
        for (let i=0; i<countSectorTwo; i++) 
            { 
            if(this.sectorTwoPlanetList[i] == system.info.systemID)
                {
                flagSectorTwo = this.sectorTwoPlanetList[i];
                } 
            }        
        }

    if((flagSectorOne >=0)||(flagSectorTwo >=0))
        {
        var tagPlanet = system.name;    // selecting Famous Planet landscape          
        this.mainFieldImage = ('fieldFP_' + tagPlanet + '.png');
        }
    else
        {
        var tagPlanet = $home_planet_makeup();  // selecting System Makeup landscape
        this.mainFieldImage = ('field_home_planet' + tagPlanet + '.png');
        }

    var planetOrbit = 0;    
    var sunDistance = player.ship.position.distanceTo(system.sun).toFixed(0);
    if(sunDistance > system.mainPlanet.radius * system.info.sun_distance_modifier)
        {
        planetOrbit = 1;
        }
    else
        {
        planetOrbit = 0;
        }

    if(system.techLevel > 13)
        {
        this.mainCityImage = "mainCity36.png";
        this.mainFieldImage = "field_home_planet128.png";
        } 

    // minor planet landscapes
    this.minorFieldImage = "oplanet_Ares.png";
    if(this.lastPlanet.radius == 30000)
        {
        this.minorFieldImage = "oplanet_Hermes.png";
        }
    if(this.lastPlanet.radius == 50000)
        {
        if(planetOrbit == 1)
            {
            this.minorFieldImage = "oplanet_Ares.png";
            }
        else
            {
            this.minorFieldImage = "oplanet_Aphrodite.png";
            }
        }
    if(this.lastPlanet.radius == 40000)
        {
        this.minorFieldImage = "oplanet_Phaethon.png";
        }
    if(this.lastPlanet.radius == 27500)
        {
        if(planetOrbit == 1)
            {
            this.minorFieldImage = "oplanet_Hades.png";
            }
        else
            {
            this.minorFieldImage = "oplanet_Hermes.png";
            }
        }

    // adjust image vs tech level
    var techImageShift = 0;
    if(system.techLevel > 3)
        {
        techImageShift = 1;
        }
    if(system.techLevel > 9)
        {
        techImageShift = 2;
        }

    // minor ports
    if((this.minorFieldImage == "oplanet_Hermes.png") || (this.minorFieldImage == "oplanet_Hades.png"))
        {
        this.minorCityImage = "minorCityLunar" + techImageShift + ".png";
        this.minorShipYardImage = "minorShipYardLunar.png";
        this.minorMineImage = "minorMineLunar.png";
        this.minorFactoryImage = "minorFactoryLunar.png";
        this.minorFarmImage = "minorFarmLunar.png";
        this.minorLabImage = "minorLabLunar" + techImageShift + ".png";
        this.minorLeisureImage = "minorLeisureLunar.png";
        this.minorDerelictCityImage = "minorDerelictCityLunar.png";
        this.minorDerelictShipYardImage = "moonDerelictShipYard.png";
        this.minorDerelictMineImage = "moonDerelictMine.png";
        this.minorDerelictFactoryImage = "moonDerelictFactory.png";
        this.minorDerelictFarmImage = "moonDerelictFarm.png";
        this.minorDerelictLeisureImage = "moonDerelictLeisure.png";
        }
    if((this.minorFieldImage == "oplanet_Ares.png") || (this.minorFieldImage == "oplanet_Phaethon.png"))
        {
        this.minorCityImage = "minorCityMartian" + techImageShift + ".png";
        this.minorShipYardImage = "minorShipYardMartian.png";
        this.minorMineImage = "minorMineMartian.png";
        this.minorFactoryImage = "minorFactoryMartian" + techImageShift + ".png";
        this.minorFarmImage = "minorFarmMartian.png";
        this.minorLabImage = "minorLabMartian" + techImageShift + ".png";
        this.minorLeisureImage = "minorLeisureMartian.png";
        this.minorDerelictCityImage = "minorDerelictCityMartian.png";
        this.minorDerelictShipYardImage = "minorDerelictShipYardMartian.png";
        this.minorDerelictMineImage = "minorDerelictMine.png";
        this.minorDerelictFactoryImage = "minorDerelictFactoryMartian.png";
        this.minorDerelictFarmImage = "minorDerelictFarm.png";
        this.minorDerelictLeisureImage = "minorDerelictLeisureMartian.png";
        }
    if(this.minorFieldImage == "oplanet_Aphrodite.png")
        {
        this.minorCityImage = "minorCityVenusian" + techImageShift + ".png";
        this.minorShipYardImage = "minorShipYardVenusian.png";
        this.minorMineImage = "minorMineVenusian.png";
        this.minorFactoryImage = "minorFactoryVenusian" + techImageShift + ".png";
        this.minorFarmImage = "minorFarmVenusian.png";
        this.minorLabImage = "minorLabVenusian.png";
        this.minorLeisureImage = "minorLeisureVenusian.png";        
        this.minorDerelictCityImage = "minorDerelictCityVenusian.png";
        this.minorDerelictShipYardImage = "minorDerelictShipYardVenusian.png";
        this.minorDerelictMineImage = "minorDerelictMine.png";
        this.minorDerelictFactoryImage = "minorDerelictFactoryVenusian.png";
        this.minorDerelictFarmImage = "minorDerelictFarm.png";
        this.minorDerelictLeisureImage = "minorDerelictLeisureVenusian.png";
        }

    // moon landscapes
    this.moonFieldImage = "omoon_Luna.png";
    var paintMoonInner = system.scrambledPseudoRandomNumber(143);
    var paintMoonOuter = system.scrambledPseudoRandomNumber(159);
    if(this.lastPlanet.radius == 17250)
        {
        this.moonFieldImage = "omoon_Selene.png";
        }
    if(this.lastPlanet.radius == 12000)
        {
        this.moonFieldImage = "omoon_Luna.png";
        }
    if(this.lastPlanet.radius == 18000)
        {
        if (paintMoonInner < 0.5)
            {
            this.moonFieldImage = "omoon_Io.png";
            }
        else
            {
            this.moonFieldImage = "omoon_Europa.png";
            }        
        }
    if(this.lastPlanet.radius == 24000)
        {
        if (paintMoonOuter < 0.5)
            {
            this.moonFieldImage = "omoon_Ganymede.png";
            }
        else
            {
            this.moonFieldImage = "omoon_Callisto.png";
            }        
        }

    // moon ports
    this.moonCityImage = "moonColony" + techImageShift + ".png";
    this.moonShipYardImage = "moonShipYard.png";
    this.moonMineImage = "moonMine" + techImageShift + ".png";
    this.moonFactoryImage = "moonFactory" + techImageShift + ".png";
    this.moonFarmImage = "moonFarm.png";
    this.moonLabImage = "moonLab" + techImageShift + ".png";
    this.moonLeisureImage = "moonLeisure" + techImageShift + ".png";
    this.moonDerelictColonyImage = "moonDerelictColony.png";
    this.moonDerelictShipYardImage = "moonDerelictShipYard.png";
    this.moonDerelictMineImage = "moonDerelictMine.png";
    this.moonDerelictFactoryImage = "moonDerelictFactory.png";
    this.moonDerelictFarmImage = "moonDerelictFarm.png";
    this.moonDerelictLeisureImage = "moonDerelictLeisure.png";    

    // gas giants
    if(this.lastPlanet.radius >= this.gasGiantRadius)
        {
        this.mainLabImage = "gasGiantStation.png";
        this.mainFieldImage = "gasGiantStation.png";
        this.minorLabImage = "gasGiantStation.png";
        this.minorFieldImage = "gasGiantStation.png";
        }

    }

// displays selected ports in the MFD
this.$updateview = function()
    {
	var p = player.ship;
	if(p && p.isValid && p.equipmentStatus("EQ_PLANETFALL") == "EQUIPMENT_OK")
        {
		var output = "";
		var glyph = ">";
		if (this.portList != null && (this.portListlength != 1 || this.portList[0] != ""))
            {
			var start = 1;
			// build the header
			output = "SELECTED PORT:";
			output += "\n";
			for (var i = start; i < this.portList.length; i++)
                {
				if (this.landingChoice == i)
                    {
					output += glyph + " ";
					if (this.outputToConsole == true)
                        {
						player.consoleMessage("Пункт назначения: " + this.portList[i]);
                        }
                    }
				output += this.portList[i] + "\n";
                }
            }
        else
            {
			output = "SELECTED PORT:";
            }
		// send output to MFD
		p.setMultiFunctionText("SelectPortMFD", output, false);
        }
    }
